#' Moderated Statistical Tests for Influence Functions
#'
#' Performs variance shrinkage via the empirical Bayes procedure of LIMMA on the
#' observed data after a transformation moving the data to influence function
#' space, based on the average treatment effect parameter.
#'
#' @param biotmle \code{biotmle} object as generated by \code{biomarkertmle}
#' @param adjust the multiple testing correction to be applied to p-values that
#'  are generated from the moderated tests. The recommended (and default) method
#'  is that of Benjamini and Hochberg. See \link[limma]{topTable} for a list of
#'  appropriate methods.
#'
#' @importFrom limma lmFit eBayes topTable
#'
#' @return \code{biotmle} object containing output from \code{limma::lmFit} and
#'  \code{limma::topTable}
#'
#' @export modtest_ic
#'
#' @examples
#' library(biotmleData)
#' library(SummarizedExperiment)
#' data(biomarkertmleOut)
#'
#' limmaTMLEout <- modtest_ic(biotmle = biomarkerTMLEout)
#
modtest_ic <- function(biotmle,
                       adjust = "BH") {
  stopifnot(class(biotmle) == "bioTMLE")
  biomarkerTMLEout <- as.data.frame(biotmle@tmleOut)

  design <- rep(1, nrow(colData(biotmle)))
  fit <- limma::lmFit(object = biomarkerTMLEout, design = design)
  fit <- limma::eBayes(fit = fit)

  tt <- limma::topTable(
    fit = fit, coef = 1, adjust.method = "BH",
    sort.by = "none", number = Inf
  )
  tt$IDs <- rownames(biomarkerTMLEout)

  biotmle@topTable <- as.data.frame(tt)
  return(biotmle)
}
